<?php
require_once 'AliZend/Form.php';

class AliForm_Admin extends AliZend_Form
{
    public function init()
    {
        require_once 'AliPumka/Filter/KeywordList.php';
        require_once 'AliPumka/Filter/UrlEncode.php';

        $this->setDisableTranslator(TRUE);

        $nonQuoteTypes = Ali_Application::getConfig()->nonQuotePostTypes->toArray();
        foreach (get_post_types(array('show_ui' => TRUE, 'show_in_nav_menus' => TRUE), 'objects') as $type=>$info) {
            $postLabel = strtolower($info->label);
            if (!in_array($type, $nonQuoteTypes)) {
                $postLabel = "'{$postLabel}'";
            }
            $multiOptions[$type] = "Enable linking in {$postLabel}";
        }

        $this->addElement('multiCheckbox', 'link_post_types', array(
            'multiOptions' => $multiOptions,
        ));

        $this->addElement('checkbox', 'link_new_window', array(
            'label' => 'Open links in new window',
        ));

        $this->addElement('checkbox', 'link_nofollow', array(
            'label' => 'Make links nofollow',
        ));

        $this->addElement('text', 'dir_path', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                new AliPumka_Filter_UrlEncode,
            ),
            'validators' => array(
            ),
            'label' => 'Redirection directory path',
        ));

        $this->addElement('text', 'max_links', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'Digits',
                'Null',
            ),
            'validators' => array(
                'Digits',
            ),
            'label' => 'Default maximum links per post/page'
        ));

        $this->addElement('textarea', 'keywords', array(
            'filters' => array(
                new AliPumka_Filter_KeywordList,
            ),
            'label' => 'Enter your keywords and respective URLs to link to',
        ));
    }


    public function isValid($data)
    {
        $this->setDefaults($data);
        $result = parent::isValid($data);

        return $result;
    }

    public function load()
    {
        /**
        * @var Ali_Model_Options
        */
        $options = Ali_Application::getModel('Options');

        foreach ($this->getElements() as $elem) {
            $elem->setValue($options->getValue($elem->getName(), NULL));
        }

        return $this;
    }

    public function save()
    {
        $linker = Ali_Application::getService('Linker');
        $linker->parseKeywordsField($this->getValue('keywords'));

        /**
        * @var Ali_Model_Options
        */
        $options = Ali_Application::getModel('Options');

        foreach ($this->getElements() as $elem) {
            $options->setValue($elem->getName(), $elem->getValue());
        }

        $linker->saveCache();

        return $this;
    }
}